; SXB Loader - by Michael Kohn
; https://www.mikekohn.net/
;
; This program will listen to the UART and copy the bytes being
; read in into RAM. When it reads a byte of 0x00, it will jmp
; to that code.

.65816

.include "w65c265.inc"

.org 0x1000
start:
  ; Disable interrupts to protect from ROM routines running.
  sei

  ; Set native mode.
  clc
  xce

  ; Set A to 8-bit
  sep #0x20

  ; Set X/Y to 16-bit
  rep #0x10

  ; UART control: receiver enable, 8 bit, transmit enable
  ;lda.b #0x25
  ;sta ACSR0
  ; Timer control (Baud Rate?)
  ;lda.b #0x17
  ;sta TCR

  ; disable interrupts on port 5
  lda.b #0x00
  sta UIER

  ;; Turn off LEDs.
  lda.b #0
  sta PD7

  ; Port 50 = DATA
  ; Port 51 = CLK
  lda.b #0
  sta PDD5

  ldy #32
  lda.b #0
memset:
  sta code, y
  dey
  bne memset

  ldy #0
main:
  lda.b #0
  sta data
  lda.b #8
  sta count
main_next_bit:
  lda PD5
  bit.b #2
  beq main_next_bit
  asl data
  bit.b #1
  beq main_next_bit_not_one
  lda data
  ora.b #1
  sta data
main_next_bit_not_one:
  ;; Wait for the clock to go low.
main_wait_clock:
  lda PD5
  bit.b #2
  bne main_wait_clock
  dec count
  bne main_next_bit
  lda data
  sta code, y
  iny
  sty total
  ;; if (data == 0) { goto code; }
  lda data
  cmp.b #0x00
  beq code
  jmp main

debug:
  lda.b #0xff
  sta PD7
debug_loop:
  bra debug_loop

read_from_uart:
  ; check for byte in UART
  lda UIFR
  bit.b #0x01
  bne main

  ;; Clear interrupt flag
  lda UIFR
  and.b #0xfe
  sta UIFR

  lda ARTD0
  rts

data:
  resb 1
count:
  resb 1
total:
  resw 1

code:

